@@ -50,6 +50,7 @@ type options struct {
 	NoConfigFile      bool
 	BinDirectory      string
 	Directory         string
+	Mode              int
 	Host              string
 	Port              int
 	Username          string
@@ -81,19 +82,28 @@ type options struct {
 	Decrypt           bool
 	WithRolePasswords bool
 	DumpOnly          bool
+	UniformTimestamp  bool
 
-	Upload       string // values are none, s3, sftp, gcs
-	Download     string // values are none, s3, sftp, gcs
-	ListRemote   string // values are none, s3, sftp, gcs
-	PurgeRemote  bool
-	S3Region     string
-	S3Bucket     string
-	S3EndPoint   string
-	S3Profile    string
-	S3KeyID      string
-	S3Secret     string
-	S3ForcePath  bool
-	S3DisableTLS bool
+	Upload         string // values are none, b2, s3, sftp, gcs
+	UploadPrefix   string
+	DeleteUploaded bool
+	Download       string // values are none, b2, s3, sftp, gcs
+	ListRemote     string // values are none, b2, s3, sftp, gcs
+	PurgeRemote    bool
+	S3Region       string
+	S3Bucket       string
+	S3EndPoint     string
+	S3Profile      string
+	S3KeyID        string
+	S3Secret       string
+	S3ForcePath    bool
+	S3DisableTLS   bool
+
+	B2Bucket                string
+	B2KeyID                 string
+	B2AppKey                string
+	B2ForcePath             bool
+	B2ConcurrentConnections int
 
 	SFTPHost             string
 	SFTPPort             string
@@ -120,23 +130,25 @@ func defaultOptions() options {
 	}
 
 	return options{
-		NoConfigFile:      false,
-		Directory:         "/var/backups/postgresql",
-		Format:            'c',
-		DirJobs:           1,
-		CompressLevel:     -1,
-		Jobs:              1,
-		PauseTimeout:      3600,
-		PurgeInterval:     -30 * 24 * time.Hour,
-		PurgeKeep:         0,
-		SumAlgo:           "none",
-		CfgFile:           defaultCfgFile,
-		TimeFormat:        timeFormat,
-		WithRolePasswords: true,
-		Upload:            "none",
-		Download:          "none",
-		ListRemote:        "none",
-		AzureEndpoint:     "blob.core.windows.net",
+		NoConfigFile:            false,
+		Directory:               "/var/backups/postgresql",
+		Mode:                    0o600,
+		Format:                  'c',
+		DirJobs:                 1,
+		CompressLevel:           -1,
+		Jobs:                    1,
+		PauseTimeout:            3600,
+		PurgeInterval:           -30 * 24 * time.Hour,
+		PurgeKeep:               0,
+		SumAlgo:                 "none",
+		CfgFile:                 defaultCfgFile,
+		TimeFormat:              timeFormat,
+		WithRolePasswords:       true,
+		Upload:                  "none",
+		Download:                "none",
+		ListRemote:              "none",
+		AzureEndpoint:           "blob.core.windows.net",
+		B2ConcurrentConnections: 5,
 	}
 }
 
@@ -153,6 +165,17 @@ func (*parseCliResult) Error() string {
 	return "please exit now"
 }
 
+func validateMode(s string) (int, error) {
+	if (strings.HasPrefix(s, "0") && len(s) <= 5) || (strings.HasPrefix(s, "-")) {
+		mode, err := strconv.ParseInt(s, 0, 32)
+		if err != nil {
+			return 0, fmt.Errorf("Invalid permission %q", s)
+		}
+		return int(mode), nil
+	}
+	return 0, fmt.Errorf("Invalid permission %q, must be octal (start by 0 and max 5 digits) number or negative", s)
+}
+
 func validateDumpFormat(s string) error {
 	for _, format := range []string{"plain", "custom", "tar", "directory"} {
 		// PostgreSQL tools allow the full name of the format and the
@@ -243,7 +266,7 @@ func validateDirectory(s string) error {
 }
 
 func parseCli(args []string) (options, []string, error) {
-	var format, purgeKeep, purgeInterval string
+	var format, mode, purgeKeep, purgeInterval string
 
 	opts := defaultOptions()
 	pce := &parseCliResult{}
@@ -257,10 +280,11 @@ func parseCli(args []string) (options, []string, error) {
 		pflag.PrintDefaults()
 	}
 
+	pflag.StringVarP(&opts.CfgFile, "config", "c", defaultCfgFile, "alternate config file")
 	pflag.BoolVar(&opts.NoConfigFile, "no-config-file", false, "skip reading config file\n")
 	pflag.StringVarP(&opts.BinDirectory, "bin-directory", "B", "", "PostgreSQL binaries directory. Empty to search $PATH")
 	pflag.StringVarP(&opts.Directory, "backup-directory", "b", "/var/backups/postgresql", "store dump files there")
-	pflag.StringVarP(&opts.CfgFile, "config", "c", defaultCfgFile, "alternate config file")
+	pflag.StringVarP(&mode, "backup-file-mode", "m", "0600", "mode to apply to dump files")
 	pflag.StringSliceVarP(&opts.ExcludeDbs, "exclude-dbs", "D", []string{}, "list of databases to exclude")
 	pflag.BoolVarP(&opts.WithTemplates, "with-templates", "t", false, "include templates")
 	WithoutTemplates := pflag.Bool("without-templates", false, "force exclude templates")
@@ -273,6 +297,7 @@ func parseCli(args []string) (options, []string, error) {
 	pflag.IntVarP(&opts.DirJobs, "parallel-backup-jobs", "J", 1, "number of parallel jobs to dumps when using directory format")
 	pflag.IntVarP(&opts.CompressLevel, "compress", "Z", -1, "compression level for compressed formats")
 	pflag.StringVarP(&opts.SumAlgo, "checksum-algo", "S", "none", "signature algorithm: none sha1 sha224 sha256 sha384 sha512")
+	pflag.BoolVar(&opts.UniformTimestamp, "uniform-timestamp", false, "Use the same timestamp for all pg_back files instead of individual\ncreation times")
 	pflag.StringVarP(&purgeInterval, "purge-older-than", "P", "30", "purge backups older than this duration in days\nuse an interval with units \"s\" (seconds), \"m\" (minutes) or \"h\" (hours)\nfor less than a day.")
 	pflag.StringVarP(&purgeKeep, "purge-min-keep", "K", "0", "minimum number of dumps to keep when purging or 'all' to keep\neverything")
 	pflag.StringVar(&opts.PreHook, "pre-backup-hook", "", "command to run before taking dumps")
@@ -283,15 +308,23 @@ func parseCli(args []string) (options, []string, error) {
 	pflag.BoolVar(&opts.EncryptKeepSrc, "encrypt-keep-src", false, "keep original files when encrypting")
 	NoEncryptKeepSrc := pflag.Bool("no-encrypt-keep-src", false, "do not keep original files when encrypting")
 	pflag.BoolVar(&opts.Decrypt, "decrypt", false, "decrypt files in the backup directory instead of dumping. DBNAMEs become\nglobs to select files")
-	pflag.StringVar(&opts.CipherPassphrase, "cipher-pass", "", "cipher passphrase for encryption and decryption\n")
-	pflag.StringVar(&opts.CipherPublicKey, "cipher-public-key", "", "AGE public key for encryption; in Bech32 encoding starting with 'age1'\n")
-	pflag.StringVar(&opts.CipherPrivateKey, "cipher-private-key", "", "AGE private key for decryption; in Bech32 encoding starting with 'AGE-SECRET-KEY-1'\n")
+	pflag.StringVar(&opts.CipherPassphrase, "cipher-pass", "", "cipher passphrase for encryption and decryption")
+	pflag.StringVar(&opts.CipherPublicKey, "cipher-public-key", "", "AGE public key for encryption; in Bech32 encoding starting with 'age1'")
+	pflag.StringVar(&opts.CipherPrivateKey, "cipher-private-key", "", "AGE private key for decryption; in Bech32 encoding starting with\n'AGE-SECRET-KEY-1'\n")
 
 	pflag.StringVar(&opts.Upload, "upload", "none", "upload produced files to target (s3, gcs,..) use \"none\" to override\nconfiguration file and disable upload")
+	pflag.StringVar(&opts.UploadPrefix, "upload-prefix", "", "add this prefix to uploaded files, similar to a target directory")
+	deleteUploaded := pflag.String("delete-uploaded", "no", "delete local file after upload")
 	pflag.StringVar(&opts.Download, "download", "none", "download files from target (s3, gcs,..) instead of dumping. DBNAMEs become\nglobs to select files")
-	pflag.StringVar(&opts.ListRemote, "list-remote", "none", "list the remote files on s3, gcs, sftp, azure instead of dumping. DBNAMEs become\nglobs to select files")
+	pflag.StringVar(&opts.ListRemote, "list-remote", "none", "list the remote files on s3, gcs, sftp, azure instead of dumping. DBNAMEs\nbecome globs to select files")
 	purgeRemote := pflag.String("purge-remote", "no", "purge the file on remote location after upload, with the same rules\nas the local directory")
 
+	pflag.StringVar(&opts.B2Bucket, "b2-bucket", "", "Backblaze B2 bucket")
+	pflag.StringVar(&opts.B2KeyID, "b2-key-id", "", "Backblaze B2 access key ID")
+	pflag.StringVar(&opts.B2AppKey, "b2-app-key", "", "Backblaze B2 app key")
+	B2ForcePath := pflag.String("b2-force-path", "no", "force path style addressing instead of virtual hosted bucket\naddressing")
+	B2ConcurrentConnections := pflag.Int("b2-concurrent-connections", 5, "set the amount of concurrent b2 http connections")
+
 	pflag.StringVar(&opts.S3Region, "s3-region", "", "S3 region")
 	pflag.StringVar(&opts.S3Bucket, "s3-bucket", "", "S3 bucket")
 	pflag.StringVar(&opts.S3Profile, "s3-profile", "", "AWS client profile name to get credentials")
@@ -399,6 +432,12 @@ func parseCli(args []string) (options, []string, error) {
 		changed = append(changed, "include-dbs")
 	}
 
+	parsed_mode, err := validateMode(mode)
+	if err != nil {
+		return opts, changed, fmt.Errorf("invalid value for --backup-file-mode: %s", err)
+	}
+	opts.Mode = parsed_mode
+
 	// Validate purge keep and time limit
 	keep, err := validatePurgeKeepValue(purgeKeep)
 	if err != nil {
@@ -445,7 +484,7 @@ func parseCli(args []string) (options, []string, error) {
 	}
 
 	// Validate upload and download options
-	stores := []string{"none", "s3", "sftp", "gcs", "azure"}
+	stores := []string{"none", "b2", "s3", "sftp", "gcs", "azure"}
 	if err := validateEnum(opts.Upload, stores); err != nil {
 		return opts, changed, fmt.Errorf("invalid value for --upload: %s", err)
 	}
@@ -458,6 +497,11 @@ func parseCli(args []string) (options, []string, error) {
 		return opts, changed, fmt.Errorf("invalid value for --list-remote: %s", err)
 	}
 
+	opts.DeleteUploaded, err = validateYesNoOption(*deleteUploaded)
+	if err != nil {
+		return opts, changed, fmt.Errorf("invalid value for --delete-uploaded: %s", err)
+	}
+
 	opts.PurgeRemote, err = validateYesNoOption(*purgeRemote)
 	if err != nil {
 		return opts, changed, fmt.Errorf("invalid value for --purge-remote: %s", err)
@@ -465,6 +509,18 @@ func parseCli(args []string) (options, []string, error) {
 
 	for _, o := range []string{opts.Upload, opts.Download, opts.ListRemote} {
 		switch o {
+		case "b2":
+			opts.B2ForcePath, err = validateYesNoOption(*B2ForcePath)
+			if err != nil {
+				return opts, changed, fmt.Errorf("invalid value for --b2-force-path: %s", err)
+			}
+
+			if *B2ConcurrentConnections <= 0 {
+				return opts, changed, fmt.Errorf("b2 concurrent connections must be more than 0 (current %d)", *B2ConcurrentConnections)
+			} else {
+				opts.B2ConcurrentConnections = *B2ConcurrentConnections
+			}
+
 		case "s3":
 			// Validate S3 options
 			opts.S3ForcePath, err = validateYesNoOption(*S3ForcePath)
@@ -491,18 +547,21 @@ func parseCli(args []string) (options, []string, error) {
 
 func validateConfigurationFile(cfg *ini.File) error {
 	s, _ := cfg.GetSection(ini.DefaultSection)
+
 	known_globals := []string{
-		"bin_directory", "backup_directory", "timestamp_format", "host", "port", "user",
+		"bin_directory", "backup_directory", "backup_file_mode", "timestamp_format", "host", "port", "user",
 		"dbname", "exclude_dbs", "include_dbs", "with_templates", "format",
 		"parallel_backup_jobs", "compress_level", "jobs", "pause_timeout",
 		"purge_older_than", "purge_min_keep", "checksum_algorithm", "pre_backup_hook",
 		"post_backup_hook", "encrypt", "cipher_pass", "cipher_public_key", "cipher_private_key",
-		"encrypt_keep_source", "upload", "purge_remote", "s3_region", "s3_bucket", "s3_endpoint",
+		"encrypt_keep_source", "upload", "purge_remote",
+		"b2_bucket", "b2_key_id", "b2_app_key", "b2_force_path",
+		"b2_concurrent_connections", "s3_region", "s3_bucket", "s3_endpoint",
 		"s3_profile", "s3_key_id", "s3_secret", "s3_force_path", "s3_tls", "sftp_host",
 		"sftp_port", "sftp_user", "sftp_password", "sftp_directory", "sftp_identity",
 		"sftp_ignore_hostkey", "gcs_bucket", "gcs_endpoint", "gcs_keyfile",
 		"azure_container", "azure_account", "azure_key", "azure_endpoint", "pg_dump_options",
-		"dump_role_passwords", "dump_only",
+		"dump_role_passwords", "dump_only", "upload_prefix", "delete_uploaded", "uniform_timestamp",
 	}
 
 gkLoop:
@@ -517,10 +576,10 @@ gkLoop:
 	}
 
 	subs := cfg.Sections()
-	knonw_perdb := []string{
+	known_perdb := []string{
 		"format", "parallel_backup_jobs", "compress_level", "checksum_algorithm",
 		"purge_older_than", "purge_min_keep", "schemas", "exclude_schemas", "tables",
-		"exclude_tables", "pg_dump_options", "with_blobs",
+		"exclude_tables", "pg_dump_options", "with_blobs", "user",
 	}
 
 	for _, sub := range subs {
@@ -530,7 +589,7 @@ gkLoop:
 
 	dbkLoop:
 		for _, v := range sub.KeyStrings() {
-			for _, c := range knonw_perdb {
+			for _, c := range known_perdb {
 				if v == c {
 					continue dbkLoop
 				}
@@ -544,7 +603,7 @@ gkLoop:
 }
 
 func loadConfigurationFile(path string) (options, error) {
-	var format, purgeKeep, purgeInterval string
+	var format, mode, purgeKeep, purgeInterval string
 
 	opts := defaultOptions()
 
@@ -570,6 +629,7 @@ func loadConfigurationFile(path string) (options, error) {
 	// flags
 	opts.BinDirectory = s.Key("bin_directory").MustString("")
 	opts.Directory = s.Key("backup_directory").MustString("/var/backups/postgresql")
+	mode = s.Key("backup_file_mode").MustString("0600")
 	timeFormat := s.Key("timestamp_format").MustString("rfc3339")
 	opts.Host = s.Key("host").MustString("")
 	opts.Port = s.Key("port").MustInt(0)
@@ -595,10 +655,19 @@ func loadConfigurationFile(path string) (options, error) {
 	opts.CipherPublicKey = s.Key("cipher_public_key").MustString("")
 	opts.CipherPrivateKey = s.Key("cipher_private_key").MustString("")
 	opts.EncryptKeepSrc = s.Key("encrypt_keep_source").MustBool(false)
+	opts.UniformTimestamp = s.Key("uniform_timestamp").MustBool(false)
 
 	opts.Upload = s.Key("upload").MustString("none")
+	opts.UploadPrefix = s.Key("upload_prefix").MustString("")
+	opts.DeleteUploaded = s.Key("delete_uploaded").MustBool(false)
 	opts.PurgeRemote = s.Key("purge_remote").MustBool(false)
 
+	opts.B2Bucket = s.Key("b2_bucket").MustString("")
+	opts.B2KeyID = s.Key("b2_key_id").MustString("")
+	opts.B2AppKey = s.Key("b2_app_key").MustString("")
+	opts.B2ForcePath = s.Key("b2_force_path").MustBool(false)
+	opts.B2ConcurrentConnections = s.Key("b2_concurrent_connections").MustInt(5)
+
 	opts.S3Region = s.Key("s3_region").MustString("")
 	opts.S3Bucket = s.Key("s3_bucket").MustString("")
 	opts.S3EndPoint = s.Key("s3_endpoint").MustString("")
@@ -625,6 +694,13 @@ func loadConfigurationFile(path string) (options, error) {
 	opts.AzureKey = s.Key("azure_key").MustString("")
 	opts.AzureEndpoint = s.Key("azure_endpoint").MustString("blob.core.windows.net")
 
+	// Validate mode and convert to int
+	m, err := validateMode(mode)
+	if err != nil {
+		return opts, err
+	}
+	opts.Mode = m
+
 	// Validate purge keep and time limit
 	keep, err := validatePurgeKeepValue(purgeKeep)
 	if err != nil {
@@ -657,8 +733,12 @@ func loadConfigurationFile(path string) (options, error) {
 		}
 	}
 
+	if opts.B2ConcurrentConnections <= 0 {
+		return opts, fmt.Errorf("b2 concurrent connections must be more than 0 (current %d)", opts.B2ConcurrentConnections)
+	}
+
 	// Validate upload option
-	stores := []string{"none", "s3", "sftp", "gcs", "azure"}
+	stores := []string{"none", "b2", "s3", "sftp", "gcs", "azure"}
 	if err := validateEnum(opts.Upload, stores); err != nil {
 		return opts, fmt.Errorf("invalid value for upload: %s", err)
 	}
@@ -704,6 +784,7 @@ func loadConfigurationFile(path string) (options, error) {
 		o.SumAlgo = s.Key("checksum_algorithm").MustString(opts.SumAlgo)
 		dbPurgeInterval = s.Key("purge_older_than").MustString(purgeInterval)
 		dbPurgeKeep = s.Key("purge_min_keep").MustString(purgeKeep)
+		o.Username = s.Key("user").MustString(opts.Username)
 
 		// Validate purge keep and time limit
 		keep, err := validatePurgeKeepValue(dbPurgeKeep)
@@ -769,6 +850,8 @@ func mergeCliAndConfigOptions(cliOpts options, configOpts options, onCli []strin
 			opts.BinDirectory = cliOpts.BinDirectory
 		case "backup-directory":
 			opts.Directory = cliOpts.Directory
+		case "backup-file-mode":
+			opts.Mode = cliOpts.Mode
 		case "exclude-dbs":
 			opts.ExcludeDbs = cliOpts.ExcludeDbs
 		case "include-dbs":
@@ -803,6 +886,9 @@ func mergeCliAndConfigOptions(cliOpts options, configOpts options, onCli []strin
 			for _, dbo := range opts.PerDbOpts {
 				dbo.SumAlgo = cliOpts.SumAlgo
 			}
+		case "uniform-timestamp":
+			opts.UniformTimestamp = cliOpts.UniformTimestamp
+
 		case "purge-older-than":
 			opts.PurgeInterval = cliOpts.PurgeInterval
 			for _, dbo := range opts.PerDbOpts {
@@ -832,12 +918,27 @@ func mergeCliAndConfigOptions(cliOpts options, configOpts options, onCli []strin
 
 		case "upload":
 			opts.Upload = cliOpts.Upload
+		case "upload-prefix":
+			opts.UploadPrefix = cliOpts.UploadPrefix
 		case "download":
 			opts.Download = cliOpts.Download
 		case "list-remote":
 			opts.ListRemote = cliOpts.ListRemote
 		case "purge-remote":
 			opts.PurgeRemote = cliOpts.PurgeRemote
+		case "delete-uploaded":
+			opts.DeleteUploaded = cliOpts.DeleteUploaded
+
+		case "b2-bucket":
+			opts.B2Bucket = cliOpts.B2Bucket
+		case "b2-key-id":
+			opts.B2KeyID = cliOpts.B2KeyID
+		case "b2-app-key":
+			opts.B2AppKey = cliOpts.B2AppKey
+		case "b2-force-path":
+			opts.B2ForcePath = cliOpts.B2ForcePath
+		case "b2-concurrent-connections":
+			opts.B2ConcurrentConnections = cliOpts.B2ConcurrentConnections
 
 		case "s3-region":
 			opts.S3Region = cliOpts.S3Region
@@ -897,6 +998,5 @@ func mergeCliAndConfigOptions(cliOpts options, configOpts options, onCli []strin
 			opts.ConnDb = cliOpts.ConnDb
 		}
 	}
-
 	return opts
 }
