@@ -62,6 +62,35 @@ func TestValidateDumpFormat(t *testing.T) {
 
 }
 
+func TestValidateMode(t *testing.T) {
+	var tests = []struct {
+		give      string
+		want      int
+		wantError bool
+	}{
+		{"0700", 448, false},
+		{"070000", 0, true},      // invalid mode (too long)
+		{"18446744000", 0, true}, // still invalid, positive integer
+		{"08170", 0, true},       // non valid mode (8 on it)
+		{"-8170", -8170, false},  // valid and mean do nothing (useful when using umask)
+	}
+
+	l.logger.SetOutput(ioutil.Discard)
+	for i, st := range tests {
+		t.Run(fmt.Sprintf("%v", i), func(t *testing.T) {
+			got, err := validateMode(st.give)
+			if err == nil && st.wantError {
+				t.Errorf("excepted an error got nil")
+			} else if err != nil && !st.wantError {
+				t.Errorf("did not want an error, got %s", err)
+			}
+			if got != st.want {
+				t.Errorf("got %v, want %v", got, st.want)
+			}
+		})
+	}
+}
+
 func TestValidatePurgeKeepValue(t *testing.T) {
 	var tests = []struct {
 		give      string
@@ -182,22 +211,24 @@ func TestDefaultOptions(t *testing.T) {
 	}
 
 	var want = options{
-		Directory:         "/var/backups/postgresql",
-		Format:            'c',
-		DirJobs:           1,
-		CompressLevel:     -1,
-		Jobs:              1,
-		PauseTimeout:      3600,
-		PurgeInterval:     -30 * 24 * time.Hour,
-		PurgeKeep:         0,
-		SumAlgo:           "none",
-		CfgFile:           "/etc/pg_back/pg_back.conf",
-		TimeFormat:        timeFormat,
-		WithRolePasswords: true,
-		Upload:            "none",
-		Download:          "none",
-		ListRemote:        "none",
-		AzureEndpoint:     "blob.core.windows.net",
+		Directory:               "/var/backups/postgresql",
+		Mode:                    0o600,
+		Format:                  'c',
+		DirJobs:                 1,
+		CompressLevel:           -1,
+		Jobs:                    1,
+		PauseTimeout:            3600,
+		PurgeInterval:           -30 * 24 * time.Hour,
+		PurgeKeep:               0,
+		SumAlgo:                 "none",
+		CfgFile:                 "/etc/pg_back/pg_back.conf",
+		TimeFormat:              timeFormat,
+		WithRolePasswords:       true,
+		Upload:                  "none",
+		Download:                "none",
+		ListRemote:              "none",
+		AzureEndpoint:           "blob.core.windows.net",
+		B2ConcurrentConnections: 5,
 	}
 
 	got := defaultOptions()
@@ -226,23 +257,25 @@ func TestParseCli(t *testing.T) {
 			{
 				[]string{"-b", "test", "-Z", "2", "a", "b"},
 				options{
-					Directory:         "test",
-					Dbnames:           []string{"a", "b"},
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     2,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					WithRolePasswords: true,
-					Upload:            "none",
-					Download:          "none",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "test",
+					Mode:                    0o600,
+					Dbnames:                 []string{"a", "b"},
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           2,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
@@ -252,23 +285,25 @@ func TestParseCli(t *testing.T) {
 			{
 				[]string{"-t", "--without-templates"},
 				options{
-					Directory:         "/var/backups/postgresql",
-					WithTemplates:     false,
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     -1,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					WithRolePasswords: true,
-					Upload:            "none",
-					Download:          "none",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					WithTemplates:           false,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
@@ -302,55 +337,59 @@ func TestParseCli(t *testing.T) {
 			{
 				[]string{"--upload", "wrong"},
 				options{
-					Directory:         "/var/backups/postgresql",
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     -1,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					Encrypt:           true,
-					CipherPassphrase:  "testpass",
-					WithRolePasswords: true,
-					Upload:            "wrong",
-					Download:          "none",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Encrypt:                 true,
+					CipherPassphrase:        "testpass",
+					WithRolePasswords:       true,
+					Upload:                  "wrong",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
-				"invalid value for --upload: value not found in [none s3 sftp gcs azure]",
+				"invalid value for --upload: value not found in [none b2 s3 sftp gcs azure]",
 				"",
 			},
 			{
 				[]string{"--download", "wrong"},
 				options{
-					Directory:         "/var/backups/postgresql",
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     -1,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					Encrypt:           true,
-					CipherPassphrase:  "testpass",
-					WithRolePasswords: true,
-					Upload:            "none",
-					Download:          "wrong",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Encrypt:                 true,
+					CipherPassphrase:        "testpass",
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "wrong",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
-				"invalid value for --download: value not found in [none s3 sftp gcs azure]",
+				"invalid value for --download: value not found in [none b2 s3 sftp gcs azure]",
 				"",
 			},
 			{
@@ -364,24 +403,26 @@ func TestParseCli(t *testing.T) {
 			{
 				[]string{"--cipher-pass", "mypass"},
 				options{
-					Directory:         "/var/backups/postgresql",
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     -1,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					Decrypt:           false,
-					CipherPassphrase:  "mypass",
-					WithRolePasswords: true,
-					Upload:            "none",
-					Download:          "none",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Decrypt:                 false,
+					CipherPassphrase:        "mypass",
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
@@ -391,24 +432,26 @@ func TestParseCli(t *testing.T) {
 			{
 				[]string{"--cipher-private-key", "mykey"},
 				options{
-					Directory:         "/var/backups/postgresql",
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     -1,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					Decrypt:           false,
-					CipherPrivateKey:  "mykey",
-					WithRolePasswords: true,
-					Upload:            "none",
-					Download:          "none",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Decrypt:                 false,
+					CipherPrivateKey:        "mykey",
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
@@ -418,24 +461,26 @@ func TestParseCli(t *testing.T) {
 			{
 				[]string{"--cipher-public-key", "fakepubkey"},
 				options{
-					Directory:         "/var/backups/postgresql",
-					Format:            'c',
-					DirJobs:           1,
-					CompressLevel:     -1,
-					Jobs:              1,
-					PauseTimeout:      3600,
-					PurgeInterval:     -30 * 24 * time.Hour,
-					PurgeKeep:         0,
-					SumAlgo:           "none",
-					CfgFile:           "/etc/pg_back/pg_back.conf",
-					TimeFormat:        timeFormat,
-					Decrypt:           false,
-					CipherPublicKey:   "fakepubkey",
-					WithRolePasswords: true,
-					Upload:            "none",
-					Download:          "none",
-					ListRemote:        "none",
-					AzureEndpoint:     "blob.core.windows.net",
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Decrypt:                 false,
+					CipherPublicKey:         "fakepubkey",
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
 				},
 				false,
 				false,
@@ -458,6 +503,74 @@ func TestParseCli(t *testing.T) {
 				"only one of --cipher-pass or --cipher-public-key allowed",
 				"",
 			},
+			{
+				[]string{"--b2-concurrent-upload", "0"},
+				defaultOptions(),
+				false,
+				false,
+				"b2 concurrent connections must be more than 0 (current 0)",
+				"",
+			},
+			{
+				[]string{"--delete-uploaded", "yes"},
+				options{
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Encrypt:                 false,
+					CipherPassphrase:        "",
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
+					DeleteUploaded:          true,
+				},
+				false,
+				false,
+				"",
+				"",
+			},
+			{
+				[]string{"--delete-uploaded", "true"},
+				options{
+					Directory:               "/var/backups/postgresql",
+					Mode:                    0o600,
+					Format:                  'c',
+					DirJobs:                 1,
+					CompressLevel:           -1,
+					Jobs:                    1,
+					PauseTimeout:            3600,
+					PurgeInterval:           -30 * 24 * time.Hour,
+					PurgeKeep:               0,
+					SumAlgo:                 "none",
+					CfgFile:                 "/etc/pg_back/pg_back.conf",
+					TimeFormat:              timeFormat,
+					Encrypt:                 false,
+					CipherPassphrase:        "",
+					WithRolePasswords:       true,
+					Upload:                  "none",
+					Download:                "none",
+					ListRemote:              "none",
+					AzureEndpoint:           "blob.core.windows.net",
+					B2ConcurrentConnections: 5,
+					DeleteUploaded:          true,
+				},
+				false,
+				false,
+				"invalid value for --delete-uploaded: value must be \"yes\" or \"no\"",
+				"",
+			},
 		}
 	)
 
@@ -522,93 +635,101 @@ func TestLoadConfigurationFile(t *testing.T) {
 		want   options
 	}{
 		{
-			[]string{"backup_directory = test", "port = 5433"},
+			[]string{"backup_directory = test", "port = 5433", "backup_file_mode = 0700"},
 			false,
 			options{
-				Directory:         "test",
-				Port:              5433,
-				Format:            'c',
-				DirJobs:           1,
-				CompressLevel:     -1,
-				Jobs:              1,
-				PauseTimeout:      3600,
-				PurgeInterval:     -30 * 24 * time.Hour,
-				PurgeKeep:         0,
-				SumAlgo:           "none",
-				CfgFile:           "/etc/pg_back/pg_back.conf",
-				TimeFormat:        timeFormat,
-				WithRolePasswords: true,
-				Upload:            "none",
-				Download:          "none",
-				ListRemote:        "none",
-				AzureEndpoint:     "blob.core.windows.net",
+				Directory:               "test",
+				Mode:                    0o700,
+				Port:                    5433,
+				Format:                  'c',
+				DirJobs:                 1,
+				CompressLevel:           -1,
+				Jobs:                    1,
+				PauseTimeout:            3600,
+				PurgeInterval:           -30 * 24 * time.Hour,
+				PurgeKeep:               0,
+				SumAlgo:                 "none",
+				CfgFile:                 "/etc/pg_back/pg_back.conf",
+				TimeFormat:              timeFormat,
+				WithRolePasswords:       true,
+				Upload:                  "none",
+				Download:                "none",
+				ListRemote:              "none",
+				AzureEndpoint:           "blob.core.windows.net",
+				B2ConcurrentConnections: 5,
 			},
 		},
 		{ // ensure comma separated lists work
-			[]string{"backup_directory = test", "include_dbs = a, b, postgres", "compress_level = 9"},
+			[]string{"backup_directory = test", "include_dbs = a, b, postgres", "compress_level = 9", "backup_file_mode = 0400"},
 			false,
 			options{
-				Directory:         "test",
-				Dbnames:           []string{"a", "b", "postgres"},
-				Format:            'c',
-				DirJobs:           1,
-				CompressLevel:     9,
-				Jobs:              1,
-				PauseTimeout:      3600,
-				PurgeInterval:     -30 * 24 * time.Hour,
-				PurgeKeep:         0,
-				SumAlgo:           "none",
-				CfgFile:           "/etc/pg_back/pg_back.conf",
-				TimeFormat:        timeFormat,
-				WithRolePasswords: true,
-				Upload:            "none",
-				Download:          "none",
-				ListRemote:        "none",
-				AzureEndpoint:     "blob.core.windows.net",
+				Directory:               "test",
+				Mode:                    0o400,
+				Dbnames:                 []string{"a", "b", "postgres"},
+				Format:                  'c',
+				DirJobs:                 1,
+				CompressLevel:           9,
+				Jobs:                    1,
+				PauseTimeout:            3600,
+				PurgeInterval:           -30 * 24 * time.Hour,
+				PurgeKeep:               0,
+				SumAlgo:                 "none",
+				CfgFile:                 "/etc/pg_back/pg_back.conf",
+				TimeFormat:              timeFormat,
+				WithRolePasswords:       true,
+				Upload:                  "none",
+				Download:                "none",
+				ListRemote:              "none",
+				AzureEndpoint:           "blob.core.windows.net",
+				B2ConcurrentConnections: 5,
 			},
 		},
 		{
 			[]string{"timestamp_format = rfc3339"},
 			false,
 			options{
-				Directory:         "/var/backups/postgresql",
-				Format:            'c',
-				DirJobs:           1,
-				CompressLevel:     -1,
-				Jobs:              1,
-				PauseTimeout:      3600,
-				PurgeInterval:     -30 * 24 * time.Hour,
-				PurgeKeep:         0,
-				SumAlgo:           "none",
-				CfgFile:           "/etc/pg_back/pg_back.conf",
-				TimeFormat:        timeFormat,
-				WithRolePasswords: true,
-				Upload:            "none",
-				Download:          "none",
-				ListRemote:        "none",
-				AzureEndpoint:     "blob.core.windows.net",
+				Directory:               "/var/backups/postgresql",
+				Mode:                    0o600,
+				Format:                  'c',
+				DirJobs:                 1,
+				CompressLevel:           -1,
+				Jobs:                    1,
+				PauseTimeout:            3600,
+				PurgeInterval:           -30 * 24 * time.Hour,
+				PurgeKeep:               0,
+				SumAlgo:                 "none",
+				CfgFile:                 "/etc/pg_back/pg_back.conf",
+				TimeFormat:              timeFormat,
+				WithRolePasswords:       true,
+				Upload:                  "none",
+				Download:                "none",
+				ListRemote:              "none",
+				AzureEndpoint:           "blob.core.windows.net",
+				B2ConcurrentConnections: 5,
 			},
 		},
 		{
 			[]string{"timestamp_format = legacy"},
 			false,
 			options{
-				Directory:         "/var/backups/postgresql",
-				Format:            'c',
-				DirJobs:           1,
-				CompressLevel:     -1,
-				Jobs:              1,
-				PauseTimeout:      3600,
-				PurgeInterval:     -30 * 24 * time.Hour,
-				PurgeKeep:         0,
-				SumAlgo:           "none",
-				CfgFile:           "/etc/pg_back/pg_back.conf",
-				TimeFormat:        "2006-01-02_15-04-05",
-				WithRolePasswords: true,
-				Upload:            "none",
-				Download:          "none",
-				ListRemote:        "none",
-				AzureEndpoint:     "blob.core.windows.net",
+				Directory:               "/var/backups/postgresql",
+				Mode:                    0o600,
+				Format:                  'c',
+				DirJobs:                 1,
+				CompressLevel:           -1,
+				Jobs:                    1,
+				PauseTimeout:            3600,
+				PurgeInterval:           -30 * 24 * time.Hour,
+				PurgeKeep:               0,
+				SumAlgo:                 "none",
+				CfgFile:                 "/etc/pg_back/pg_back.conf",
+				TimeFormat:              "2006-01-02_15-04-05",
+				WithRolePasswords:       true,
+				Upload:                  "none",
+				Download:                "none",
+				ListRemote:              "none",
+				AzureEndpoint:           "blob.core.windows.net",
+				B2ConcurrentConnections: 5,
 			},
 		},
 		{
@@ -634,6 +755,7 @@ func TestLoadConfigurationFile(t *testing.T) {
 			false,
 			options{
 				Directory:     "test",
+				Mode:          0o600,
 				Format:        'c',
 				DirJobs:       1,
 				CompressLevel: -1,
@@ -655,11 +777,12 @@ func TestLoadConfigurationFile(t *testing.T) {
 					PgDumpOpts:    []string{"-O", "-x"},
 					WithBlobs:     1,
 				}},
-				WithRolePasswords: true,
-				Upload:            "none",
-				Download:          "none",
-				ListRemote:        "none",
-				AzureEndpoint:     "blob.core.windows.net",
+				WithRolePasswords:       true,
+				Upload:                  "none",
+				Download:                "none",
+				ListRemote:              "none",
+				AzureEndpoint:           "blob.core.windows.net",
+				B2ConcurrentConnections: 5,
 			},
 		},
 		{
@@ -677,6 +800,7 @@ func TestLoadConfigurationFile(t *testing.T) {
 			false,
 			options{
 				Directory:     "test",
+				Mode:          0o600,
 				Format:        'c',
 				DirJobs:       1,
 				CompressLevel: 3,
@@ -698,13 +822,19 @@ func TestLoadConfigurationFile(t *testing.T) {
 					PgDumpOpts:    []string{},
 					WithBlobs:     2,
 				}},
-				WithRolePasswords: false,
-				Upload:            "none",
-				Download:          "none",
-				ListRemote:        "none",
-				AzureEndpoint:     "blob.core.windows.net",
+				WithRolePasswords:       false,
+				Upload:                  "none",
+				Download:                "none",
+				ListRemote:              "none",
+				AzureEndpoint:           "blob.core.windows.net",
+				B2ConcurrentConnections: 5,
 			},
 		},
+		{
+			[]string{"b2_concurrent_connections = 0"},
+			true,
+			defaultOptions(),
+		},
 	}
 
 	for i, st := range tests {
@@ -745,32 +875,34 @@ func TestMergeCliAndConfigoptions(t *testing.T) {
 	}
 
 	want := options{
-		BinDirectory:      "/bin",
-		Directory:         "test",
-		Host:              "localhost",
-		Port:              5433,
-		Username:          "test",
-		ConnDb:            "postgres",
-		ExcludeDbs:        []string{"a", "b"},
-		Dbnames:           []string{"b", "c", "d"},
-		WithTemplates:     true,
-		Format:            'd',
-		DirJobs:           2,
-		CompressLevel:     4,
-		Jobs:              4,
-		PauseTimeout:      60,
-		PurgeInterval:     -7 * 24 * time.Hour,
-		PurgeKeep:         5,
-		SumAlgo:           "sha256",
-		PreHook:           "touch /tmp/pre-hook",
-		PostHook:          "touch /tmp/post-hook",
-		CfgFile:           "/etc/pg_back/pg_back.conf",
-		TimeFormat:        timeFormat,
-		WithRolePasswords: true,
-		Upload:            "none",
-		Download:          "none",
-		ListRemote:        "none",
-		AzureEndpoint:     "blob.core.windows.net",
+		BinDirectory:            "/bin",
+		Directory:               "test",
+		Mode:                    0o600,
+		Host:                    "localhost",
+		Port:                    5433,
+		Username:                "test",
+		ConnDb:                  "postgres",
+		ExcludeDbs:              []string{"a", "b"},
+		Dbnames:                 []string{"b", "c", "d"},
+		WithTemplates:           true,
+		Format:                  'd',
+		DirJobs:                 2,
+		CompressLevel:           4,
+		Jobs:                    4,
+		PauseTimeout:            60,
+		PurgeInterval:           -7 * 24 * time.Hour,
+		PurgeKeep:               5,
+		SumAlgo:                 "sha256",
+		PreHook:                 "touch /tmp/pre-hook",
+		PostHook:                "touch /tmp/post-hook",
+		CfgFile:                 "/etc/pg_back/pg_back.conf",
+		TimeFormat:              timeFormat,
+		WithRolePasswords:       true,
+		Upload:                  "none",
+		Download:                "none",
+		ListRemote:              "none",
+		AzureEndpoint:           "blob.core.windows.net",
+		B2ConcurrentConnections: 5,
 	}
 
 	cliOptList := []string{
