@@ -32,12 +32,13 @@ import (
 	"os/exec"
 	"path/filepath"
 	"runtime"
+	"slices"
 	"strings"
 	"sync"
 	"time"
 )
 
-var version = "2.3.1"
+var version = "2.6.0"
 var binDir string
 
 type dump struct {
@@ -55,6 +56,9 @@ type dump struct {
 	// Directory is the target directory where to create the dump
 	Directory string
 
+	// Mode is the permission for the resulting backup
+	Mode int
+
 	// Time format for the filename
 	TimeFormat string
 
@@ -111,6 +115,9 @@ type dbOpts struct {
 	// -B, or let pg_dump use its default. 0 means default, 1 include
 	// blobs, 2 exclude blobs.
 	WithBlobs int
+
+	// Connection user for that database
+	Username string
 }
 
 func main() {
@@ -177,6 +184,10 @@ func run() (retVal error) {
 		return fmt.Errorf("a bucket is mandatory with s3")
 	}
 
+	if (opts.Upload == "b2" || opts.Download == "b2" || opts.ListRemote == "b2") && opts.B2Bucket == "" {
+		return fmt.Errorf("a bucket is mandatory with B2")
+	}
+
 	if (opts.Upload == "gcs" || opts.Download == "gcs" || opts.ListRemote == "gcs") && opts.GCSBucket == "" {
 		return fmt.Errorf("a bucket is mandatory with gcs")
 	}
@@ -190,11 +201,8 @@ func run() (retVal error) {
 	// config file since we are using the remaining args from the command
 	// line that are usually as a list of databases to dump
 	globs := []string{}
-	for _, v := range cliOptList {
-		if v == "include-dbs" {
-			globs = opts.Dbnames
-			break
-		}
+	if slices.Contains(cliOptList, "include-dbs") {
+		globs = opts.Dbnames
 	}
 
 	// Listing remote files take priority over the other options that won't dump databases
@@ -295,6 +303,12 @@ func run() (retVal error) {
 	}
 	defer db.Close()
 
+	// Generate a single datetime that will be used in all files generated by pg_back
+	var fileTime time.Time
+	if opts.UniformTimestamp {
+		fileTime = time.Now()
+	}
+
 	if !opts.DumpOnly {
 		if !db.superuser {
 			l.Infoln("connection user is not superuser, some information will not be dumped")
@@ -308,7 +322,7 @@ func run() (retVal error) {
 		} else {
 			l.Infoln("dumping globals without role passwords")
 		}
-		if err := dumpGlobals(opts.Directory, opts.TimeFormat, dumpRolePasswords, conninfo, producedFiles); err != nil {
+		if err := dumpGlobals(opts.Directory, opts.Mode, opts.TimeFormat, dumpRolePasswords, conninfo, producedFiles, fileTime); err != nil {
 			return fmt.Errorf("pg_dumpall of globals failed: %w", err)
 		}
 
@@ -318,7 +332,7 @@ func run() (retVal error) {
 			perr *pgPrivError
 		)
 
-		if err := dumpSettings(opts.Directory, opts.TimeFormat, db, producedFiles); err != nil {
+		if err := dumpSettings(opts.Directory, opts.Mode, opts.TimeFormat, db, producedFiles, fileTime); err != nil {
 			if errors.As(err, &verr) || errors.As(err, &perr) {
 				l.Warnln(err)
 			} else {
@@ -326,7 +340,7 @@ func run() (retVal error) {
 			}
 		}
 
-		if err := dumpConfigFiles(opts.Directory, opts.TimeFormat, db, producedFiles); err != nil {
+		if err := dumpConfigFiles(opts.Directory, opts.Mode, opts.TimeFormat, db, producedFiles, fileTime); err != nil {
 			return fmt.Errorf("could not dump configuration files: %w", err)
 		}
 	}
@@ -349,8 +363,8 @@ func run() (retVal error) {
 
 	// start workers - thanks gobyexample.com
 	l.Verbosef("launching %d workers", maxWorkers)
-	for w := 0; w < maxWorkers; w++ {
-		go dumper(w, jobs, results, producedFiles)
+	for range maxWorkers {
+		go dumper(jobs, results, producedFiles)
 	}
 
 	defDbOpts := defaultDbOpts(opts)
@@ -372,11 +386,13 @@ func run() (retVal error) {
 			Database:         dbname,
 			Options:          o,
 			Directory:        opts.Directory,
+			Mode:             opts.Mode,
 			TimeFormat:       opts.TimeFormat,
 			ConnString:       conninfo,
 			CipherPassphrase: passphrase,
 			CipherPublicKey:  publicKey,
 			EncryptKeepSrc:   opts.EncryptKeepSrc,
+			When:             fileTime,
 			ExitCode:         -1,
 			PgDumpVersion:    pgDumpVersion,
 		}
@@ -397,7 +413,7 @@ func run() (retVal error) {
 	}
 
 	// collect the result of the jobs
-	for j := 0; j < numJobs; j++ {
+	for range numJobs {
 		var b, c string
 		var err error
 
@@ -468,9 +484,8 @@ func run() (retVal error) {
 			fmt.Fprintf(f, "%s", c)
 
 			f.Close()
-
-			if err := os.Chmod(aclpath, 0600); err != nil {
-				return fmt.Errorf("could not chmod to more secure permission for ACL %s: %s", dbname, err)
+			if err := os.Chmod(aclpath, os.FileMode(d.Mode)); err != nil {
+				return fmt.Errorf("could not chmod to more secure permission for ACL %s: %w", dbname, err)
 			}
 
 			// Have its checksum computed
@@ -513,6 +528,11 @@ func run() (retVal error) {
 		if err != nil {
 			return fmt.Errorf("failed to prepare upload to S3: %w", err)
 		}
+	case "b2":
+		repo, err = NewB2Repo(opts)
+		if err != nil {
+			return fmt.Errorf("failed to prepare upload to B2: %w", err)
+		}
 	case "sftp":
 		repo, err = NewSFTPRepo(opts)
 		if err != nil {
@@ -542,22 +562,24 @@ func run() (retVal error) {
 		}
 
 		if opts.PurgeRemote && repo != nil {
-			if err := purgeRemoteDumps(repo, opts.Directory, dbname, o.PurgeKeep, limit); err != nil {
+			if err := purgeRemoteDumps(repo, opts.UploadPrefix, opts.Directory, dbname, o.PurgeKeep, limit); err != nil {
 				retVal = err
 			}
 		}
 	}
 
-	for _, other := range []string{"pg_globals", "pg_settings", "hba_file", "ident_file"} {
-		limit := now.Add(defDbOpts.PurgeInterval)
-		if err := purgeDumps(opts.Directory, other, defDbOpts.PurgeKeep, limit); err != nil {
-			retVal = err
-		}
-
-		if opts.PurgeRemote && repo != nil {
-			if err := purgeRemoteDumps(repo, opts.Directory, other, defDbOpts.PurgeKeep, limit); err != nil {
+	if !opts.DumpOnly {
+		for _, other := range []string{"pg_globals", "pg_settings", "hba_file", "ident_file"} {
+			limit := now.Add(defDbOpts.PurgeInterval)
+			if err := purgeDumps(opts.Directory, other, defDbOpts.PurgeKeep, limit); err != nil {
 				retVal = err
 			}
+
+			if opts.PurgeRemote && repo != nil {
+				if err := purgeRemoteDumps(repo, opts.UploadPrefix, opts.Directory, other, defDbOpts.PurgeKeep, limit); err != nil {
+					retVal = err
+				}
+			}
 		}
 	}
 
@@ -573,6 +595,7 @@ func defaultDbOpts(opts options) *dbOpts {
 		PurgeInterval: opts.PurgeInterval,
 		PurgeKeep:     opts.PurgeKeep,
 		PgDumpOpts:    opts.PgDumpOpts,
+		Username:      opts.Username,
 	}
 	return &dbo
 }
@@ -597,8 +620,6 @@ func (d *dump) dump(fc chan<- sumFileJob) error {
 		return fmt.Errorf("could not acquire lock for %s", dbname)
 	}
 
-	d.When = time.Now()
-
 	var fileEnd string
 	switch d.Options.Format {
 	case 'p':
@@ -615,6 +636,10 @@ func (d *dump) dump(fc chan<- sumFileJob) error {
 		fileEnd = "d"
 	}
 
+	if d.When.IsZero() {
+		d.When = time.Now()
+	}
+
 	file := formatDumpPath(d.Directory, d.TimeFormat, fileEnd, dbname, d.When, d.Options.CompressLevel)
 	formatOpt := fmt.Sprintf("-F%c", d.Options.Format)
 
@@ -671,6 +696,9 @@ func (d *dump) dump(fc chan<- sumFileJob) error {
 	// on the command line. For older version, it is passed using the
 	// environment
 	conninfo := d.ConnString.Set("dbname", dbname)
+	if d.Options.Username != "" {
+		conninfo = conninfo.Set("user", d.Options.Username)
+	}
 
 	var env []string
 
@@ -687,7 +715,7 @@ func (d *dump) dump(fc chan<- sumFileJob) error {
 	l.Verboseln("running:", pgDumpCmd)
 	stdoutStderr, err := pgDumpCmd.CombinedOutput()
 	if err != nil {
-		for _, line := range strings.Split(string(stdoutStderr), "\n") {
+		for line := range strings.SplitSeq(string(stdoutStderr), "\n") {
 			if line != "" {
 				l.Errorf("[%s] %s\n", dbname, line)
 			}
@@ -699,7 +727,7 @@ func (d *dump) dump(fc chan<- sumFileJob) error {
 		return err
 	}
 	if len(stdoutStderr) > 0 {
-		for _, line := range strings.Split(string(stdoutStderr), "\n") {
+		for line := range strings.SplitSeq(string(stdoutStderr), "\n") {
 			if line != "" {
 				l.Infof("[%s] %s\n", dbname, line)
 			}
@@ -722,21 +750,57 @@ func (d *dump) dump(fc chan<- sumFileJob) error {
 	d.Path = file
 	d.ExitCode = 0
 
-	var mode os.FileMode = 0600
-	if d.Options.Format == 'd' {
-		// The hardening of permissions only apply to the top level
-		// directory, this won't make the contents executable
-		mode = 0700
-	}
+	if d.Mode > 0 {
+		var mode os.FileMode = os.FileMode(d.Mode)
+		isDirFormat := d.Options.Format == 'd'
+		if isDirFormat {
+			// calculate appropriate permission for parent directory, we need +x
+			// permission to walk through the directory
+			if (mode&0o400 > 0) || (mode&0o200 > 0) {
+				mode = mode | 0o100
+			}
 
-	if err := os.Chmod(file, mode); err != nil {
-		return fmt.Errorf("could not chmod to more secure permission for %s: %s", dbname, err)
+			if (mode&0o040 > 0) || (mode&0o020 > 0) {
+				mode = mode | 0o010
+			}
+
+			if (mode&0o004 > 0) || (mode&0o002 > 0) {
+				mode = mode | 0o001
+			}
+		}
+
+		if err := os.Chmod(file, mode); err != nil {
+			return fmt.Errorf("could not chmod to more secure permission for %s: %w", dbname, err)
+		}
+
+		if isDirFormat {
+			// adapt mode on files on directory based on initial configured mode
+			if err := recursiveChmod(file, os.FileMode(d.Mode)); err != nil {
+				return err
+			}
+		}
 	}
 
 	return nil
 }
 
-func dumper(id int, jobs <-chan *dump, results chan<- *dump, fc chan<- sumFileJob) {
+func recursiveChmod(file string, newMode os.FileMode) error {
+	err := filepath.Walk(file, func(path string, info os.FileInfo, err error) error {
+		if err != nil {
+			return fmt.Errorf("error when accesing file %s: %w", path, err)
+		}
+		if !info.IsDir() {
+			if err := os.Chmod(path, newMode); err != nil {
+				return fmt.Errorf("could not chmod %s to more secure permission: %w", file, err)
+			}
+		}
+		return nil
+
+	})
+	return err
+}
+
+func dumper(jobs <-chan *dump, results chan<- *dump, fc chan<- sumFileJob) {
 	for j := range jobs {
 
 		if err := j.dump(fc); err != nil {
@@ -880,7 +944,7 @@ func pgToolVersion(tool string) int {
 	return numver
 }
 
-func dumpGlobals(dir string, timeFormat string, withRolePasswords bool, conninfo *ConnInfo, fc chan<- sumFileJob) error {
+func dumpGlobals(dir string, mode int, timeFormat string, withRolePasswords bool, conninfo *ConnInfo, fc chan<- sumFileJob, when time.Time) error {
 	command := execPath("pg_dumpall")
 	args := []string{"-g", "-w"}
 
@@ -912,7 +976,11 @@ func dumpGlobals(dir string, timeFormat string, withRolePasswords bool, conninfo
 		args = append(args, "--no-role-passwords")
 	}
 
-	file := formatDumpPath(dir, timeFormat, "sql", "pg_globals", time.Now(), 0)
+	if when.IsZero() {
+		when = time.Now()
+	}
+
+	file := formatDumpPath(dir, timeFormat, "sql", "pg_globals", when, 0)
 	args = append(args, "-f", file)
 
 	if err := os.MkdirAll(filepath.Dir(file), 0700); err != nil {
@@ -924,7 +992,7 @@ func dumpGlobals(dir string, timeFormat string, withRolePasswords bool, conninfo
 	l.Verboseln("running:", pgDumpallCmd)
 	stdoutStderr, err := pgDumpallCmd.CombinedOutput()
 	if err != nil {
-		for _, line := range strings.Split(string(stdoutStderr), "\n") {
+		for line := range strings.SplitSeq(string(stdoutStderr), "\n") {
 			if line != "" {
 				l.Errorln(line)
 			}
@@ -932,15 +1000,16 @@ func dumpGlobals(dir string, timeFormat string, withRolePasswords bool, conninfo
 		return err
 	}
 	if len(stdoutStderr) > 0 {
-		for _, line := range strings.Split(string(stdoutStderr), "\n") {
+		for line := range strings.SplitSeq(string(stdoutStderr), "\n") {
 			if line != "" {
 				l.Infoln(line)
 			}
 		}
 	}
-
-	if err := os.Chmod(file, 0600); err != nil {
-		return fmt.Errorf("could not chmod to more secure permission for pg_globals: %s", err)
+	if mode > 0 {
+		if err := os.Chmod(file, os.FileMode(mode)); err != nil {
+			return fmt.Errorf("could not chmod to more secure permission for pg_globals: %w", err)
+		}
 	}
 
 	if fc != nil {
@@ -952,11 +1021,14 @@ func dumpGlobals(dir string, timeFormat string, withRolePasswords bool, conninfo
 	return nil
 }
 
-func dumpSettings(dir string, timeFormat string, db *pg, fc chan<- sumFileJob) error {
+func dumpSettings(dir string, mode int, timeFormat string, db *pg, fc chan<- sumFileJob, when time.Time) error {
+	if when.IsZero() {
+		when = time.Now()
+	}
 
-	file := formatDumpPath(dir, timeFormat, "out", "pg_settings", time.Now(), 0)
+	file := formatDumpPath(dir, timeFormat, "out", "pg_settings", when, 0)
 
-	if err := os.MkdirAll(filepath.Dir(file), 0700); err != nil {
+	if err := os.MkdirAll(filepath.Dir(file), 0o700); err != nil {
 		return err
 	}
 
@@ -965,12 +1037,18 @@ func dumpSettings(dir string, timeFormat string, db *pg, fc chan<- sumFileJob) e
 		return err
 	}
 
-	// Use a Buffer to avoid creating an empty file
 	if len(s) > 0 {
 		l.Verboseln("writing settings to:", file)
-		if err := os.WriteFile(file, []byte(s), 0600); err != nil {
+		f, err := os.Create(file)
+		if err != nil {
 			return err
 		}
+		fmt.Fprintf(f, "%s", s)
+		if mode > 0 { // otherwhise let umask do the job
+			if err := os.Chmod(f.Name(), os.FileMode(mode)); err != nil {
+				return fmt.Errorf("could not chmod to more secure permission for settings file %s: %w", f.Name(), err)
+			}
+		}
 
 		if fc != nil {
 			fc <- sumFileJob{
@@ -982,9 +1060,12 @@ func dumpSettings(dir string, timeFormat string, db *pg, fc chan<- sumFileJob) e
 	return nil
 }
 
-func dumpConfigFiles(dir string, timeFormat string, db *pg, fc chan<- sumFileJob) error {
+func dumpConfigFiles(dir string, mode int, timeFormat string, db *pg, fc chan<- sumFileJob, when time.Time) error {
 	for _, param := range []string{"hba_file", "ident_file"} {
-		file := formatDumpPath(dir, timeFormat, "out", param, time.Now(), 0)
+		if when.IsZero() {
+			when = time.Now()
+		}
+		file := formatDumpPath(dir, timeFormat, "out", param, when, 0)
 
 		if err := os.MkdirAll(filepath.Dir(file), 0700); err != nil {
 			return err
@@ -998,9 +1079,16 @@ func dumpConfigFiles(dir string, timeFormat string, db *pg, fc chan<- sumFileJob
 		// Use a Buffer to avoid creating an empty file
 		if len(s) > 0 {
 			l.Verbosef("writing contents of '%s' to: %s", param, file)
-			if err := os.WriteFile(file, []byte(s), 0600); err != nil {
+			f, err := os.Create(file)
+			if err != nil {
 				return err
 			}
+			fmt.Fprintf(f, "%s", s)
+			if mode > 0 { // otherwhise let umask do the job
+				if err := os.Chmod(f.Name(), os.FileMode(mode)); err != nil {
+					return fmt.Errorf("could not chmod to more secure permission for settings file %s: %w", f.Name(), err)
+				}
+			}
 
 			// We have produced a file send it to the channel for
 			// further processing
@@ -1120,7 +1208,7 @@ func decryptDirectory(dir string, params decryptParams, workers int, globs []str
 
 	// Start workers that listen for filenames to decrypt until the queue
 	// is closed
-	for i := 0; i < workers; i++ {
+	for i := range workers {
 		wg.Add(1)
 		go func(id int) {
 			l.Verboseln("started decrypt worker", id)
@@ -1306,7 +1394,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 	encIn := make(chan encryptFileJob)
 	uploadIn := make(chan uploadJob)
 
-	for i := 0; i < opts.Jobs; i++ {
+	for i := range opts.Jobs {
 		wg.Add(1)
 		go func(id int) {
 			l.Verboseln("started checksum worker", id)
@@ -1330,7 +1418,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 
 				if j.SumAlgo != "none" {
 					l.Infoln("computing checksum of", j.Path)
-					p, err := checksumFile(j.Path, j.SumAlgo)
+					p, err := checksumFile(j.Path, opts.Mode, j.SumAlgo)
 					if err != nil {
 						l.Errorln("checksum failed:", err)
 						if !failed {
@@ -1409,7 +1497,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 
 	sumEncIn := make(chan sumEncryptFileJob)
 
-	for i := 0; i < opts.Jobs; i++ {
+	for i := range opts.Jobs {
 		wg.Add(1)
 		go func(id int) {
 			l.Verboseln("started encryption worker", id)
@@ -1425,7 +1513,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 
 				if opts.Encrypt {
 					l.Infoln("encrypting", j.Path)
-					encFiles, err := encryptFile(j.Path, j.Params, j.KeepSrc)
+					encFiles, err := encryptFile(j.Path, opts.Mode, j.Params, j.KeepSrc)
 					if err != nil {
 						l.Errorln("encryption failed:", err)
 						if !failed {
@@ -1455,7 +1543,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 		}(i)
 	}
 
-	for i := 0; i < opts.Jobs; i++ {
+	for i := range opts.Jobs {
 		wg.Add(1)
 		go func(id int) {
 			l.Verboseln("started checksum worker for encrypted files", id)
@@ -1475,7 +1563,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 
 				if j.SumAlgo != "none" {
 					l.Infoln("computing checksum of", j.SumFile)
-					p, err := checksumFileList(j.Paths, j.SumAlgo, j.SumFile)
+					p, err := checksumFileList(j.Paths, opts.Mode, j.SumAlgo, j.SumFile)
 					if err != nil {
 						l.Errorln("checksum of encrypted files failed:", err)
 						if !failed {
@@ -1503,7 +1591,7 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 		repo = nil
 	}
 
-	for i := 0; i < opts.Jobs; i++ {
+	for i := range opts.Jobs {
 		wg.Add(1)
 		go func(id int) {
 			l.Verboseln("started upload worker", id)
@@ -1518,7 +1606,8 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 				}
 
 				if opts.Upload != "none" && repo != nil {
-					if err := repo.Upload(j.Path, relPath(opts.Directory, j.Path)); err != nil {
+					// Prepend the global prefix to the relative path of the dump
+					if err := repo.Upload(j.Path, filepath.Join(opts.UploadPrefix, relPath(opts.Directory, j.Path))); err != nil {
 						l.Errorln(err)
 						if !failed {
 							ret <- err
@@ -1526,6 +1615,12 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 						}
 						continue
 					}
+
+					l.Infoln("uploaded", j.Path)
+					if opts.DeleteUploaded {
+						l.Infoln("removing", j.Path)
+						os.Remove(j.Path)
+					}
 				}
 			}
 		}(i)
@@ -1538,22 +1633,22 @@ func postProcessFiles(inFiles chan sumFileJob, wg *sync.WaitGroup, opts options)
 		// inFiles will be closed outside of the function, when all
 		// worker reading from it exit, close encIn to make the workers
 		// reading from it stop, and so on.
-		for i := 0; i < opts.Jobs; i++ {
+		for range opts.Jobs {
 			<-done
 		}
 		close(encIn)
 
-		for i := 0; i < opts.Jobs; i++ {
+		for range opts.Jobs {
 			<-done
 		}
 		close(sumEncIn)
 
-		for i := 0; i < opts.Jobs; i++ {
+		for range opts.Jobs {
 			<-done
 		}
 		close(uploadIn)
 
-		for i := 0; i < opts.Jobs; i++ {
+		for range opts.Jobs {
 			<-done
 		}
 
