@@ -26,11 +26,20 @@
 package main
 
 import (
-	"cloud.google.com/go/storage"
 	"context"
 	"errors"
 	"fmt"
+	"io"
+	"net"
+	"os"
+	"os/user"
+	"path/filepath"
+	"strings"
+	"time"
+
+	"cloud.google.com/go/storage"
 	"github.com/Azure/azure-sdk-for-go/sdk/storage/azblob"
+	"github.com/Backblaze/blazer/b2"
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/credentials"
 	"github.com/aws/aws-sdk-go/aws/session"
@@ -42,13 +51,6 @@ import (
 	"golang.org/x/crypto/ssh/knownhosts"
 	"google.golang.org/api/iterator"
 	"google.golang.org/api/option"
-	"io"
-	"net"
-	"os"
-	"os/user"
-	"path/filepath"
-	"strings"
-	"time"
 )
 
 // A Repo is a remote service where we can upload files
@@ -93,6 +95,11 @@ func NewRepo(kind string, opts options) (Repo, error) {
 		if err != nil {
 			return nil, fmt.Errorf("failed to prepare S3 repo: %w", err)
 		}
+	case "b2":
+		repo, err = NewB2Repo(opts)
+		if err != nil {
+			return nil, fmt.Errorf("failed to prepare B2 repo: %w", err)
+		}
 	case "sftp":
 		repo, err = NewSFTPRepo(opts)
 		if err != nil {
@@ -113,6 +120,17 @@ func NewRepo(kind string, opts options) (Repo, error) {
 	return repo, nil
 }
 
+type b2repo struct {
+	appKey                string
+	b2Bucket              *b2.Bucket
+	b2Client              *b2.Client
+	bucket                string
+	concurrentConnections int
+	ctx                   context.Context
+	forcePath             bool
+	keyID                 string
+}
+
 type s3repo struct {
 	region     string
 	bucket     string
@@ -125,6 +143,118 @@ type s3repo struct {
 	session    *session.Session
 }
 
+func NewB2Repo(opts options) (*b2repo, error) {
+	r := &b2repo{
+		appKey:                opts.B2AppKey,
+		bucket:                opts.B2Bucket,
+		concurrentConnections: opts.B2ConcurrentConnections,
+		ctx:                   context.Background(),
+		forcePath:             opts.B2ForcePath,
+		keyID:                 opts.B2KeyID,
+	}
+
+	l.Verbosef("starting b2 client with %d connections to endpoint to bucket %s \n", r.concurrentConnections, r.bucket)
+	client, err := b2.NewClient(r.ctx, r.keyID, r.appKey)
+
+	if err != nil {
+		return nil, fmt.Errorf("could not create B2 session: %w", err)
+	}
+
+	r.b2Client = client
+
+	bucket, err := r.b2Client.Bucket(r.ctx, r.bucket)
+
+	if err != nil {
+		return nil, fmt.Errorf("could not connect to B2 bucket: %w", err)
+	}
+
+	r.b2Bucket = bucket
+
+	return r, nil
+}
+
+func (r *b2repo) Upload(path string, target string) error {
+	f, err := os.Open(path)
+	if err != nil {
+		return err
+	}
+	defer f.Close()
+
+	w := r.b2Bucket.Object(target).NewWriter(r.ctx)
+	defer w.Close()
+
+	w.ConcurrentUploads = r.concurrentConnections
+
+	l.Infof("uploading %s to B2 bucket %s\n", path, r.bucket)
+	if _, err := io.Copy(w, f); err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func (r *b2repo) Download(target string, path string) error {
+	f, err := os.Create(path)
+	if err != nil {
+		return fmt.Errorf("download error: %w", err)
+	}
+	defer f.Close()
+
+	bucket := r.b2Bucket
+
+	l.Infof("downloading %s from B2 bucket %s to %s\n", target, r.bucket, path)
+
+	rf := bucket.Object(target).NewReader(r.ctx)
+	rf.ConcurrentDownloads = r.concurrentConnections
+	defer rf.Close()
+
+	if err != nil {
+		return err
+	}
+
+	if _, err := io.Copy(f, rf); err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func (r *b2repo) Close() error {
+	return nil
+}
+
+func (r *b2repo) List(prefix string) ([]Item, error) {
+
+	files := make([]Item, 0)
+
+	i := r.b2Bucket.List(r.ctx, b2.ListPrefix(prefix))
+	for i.Next() {
+		obj := i.Object()
+
+		attributes, err := obj.Attrs(r.ctx)
+
+		if err != nil {
+			return nil, err
+		}
+
+		files = append(files, Item{
+			key:     obj.Name(),
+			modtime: attributes.LastModified,
+		},
+		)
+	}
+
+	return files, i.Err()
+}
+
+func (r *b2repo) Remove(path string) error {
+	ctx, cancel := context.WithCancel(r.ctx)
+
+	defer cancel()
+
+	return r.b2Bucket.Object(path).Delete(ctx)
+}
+
 func NewS3Repo(opts options) (*s3repo, error) {
 	r := &s3repo{
 		region:     opts.S3Region,
@@ -773,7 +903,7 @@ func NewAzRepo(opts options) (*azRepo, error) {
 			return nil, fmt.Errorf("could not setup Azure credentials: %w", err)
 		}
 
-		url := fmt.Sprintf("https://%s.%s/%s", r.account, r.endpoint, r.container)
+		url := fmt.Sprintf("https://%s.%s", r.account, r.endpoint)
 
 		client, err = azblob.NewClientWithSharedKeyCredential(url, credential, nil)
 		if err != nil {
